(function(d){d['pt']=Object.assign(d['pt']||{},{a:"Não foi possível carregar o ficheiro:",b:"Block quote",c:"Itálico",d:"Negrito",e:"módulo de imagem",f:"Imagem em tamanho completo",g:"Imagem lateral",h:"Left aligned image",i:"Imagem centrada",j:"Right aligned image",k:"Indicar legenda da imagem",l:"Inserir imagem",m:"Hiperligação",n:"Falha ao carregar",o:"Choose heading",p:"Título",q:"Lista ordenada",r:"Lista não ordenada",s:"Change image text alternative",t:"Parágrafo",u:"Título 1",v:"Título 2",w:"Título 3",x:"Desligar",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Editor de texto avançado, %0",ac:"Editor de texto avançado",ad:"Desfazer",ae:"Refazer",af:"Guardar",ag:"Cancelar",ah:"Texto alternativo",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));