(function(d){d['fi']=Object.assign(d['fi']||{},{a:"Tiedostoa ei voitu ladata:",b:"Lainaus",c:"Kursivointi",d:"Lihavointi",e:"Kuvavimpain",f:"Täysikokoinen kuva",g:"Pieni kuva",h:"Vasemmalle tasattu kuva",i:"Keskitetty kuva",j:"Oikealle tasattu kuva",k:"Syötä kuvateksti",l:"Lisää kuva",m:"Linkki",n:"Lataus epäonnistui",o:"Valitse tyyli",p:"Otsikkotyyli",q:"Numeroitu lista",r:"Lista",s:"Vaihda kuvan vaihtoehtoinen teksti",t:"Kappale",u:"Otsikko 1",v:"Otsikko 2",w:"Otsikko 3",x:"Poista linkki",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Rikas tekstieditori, %0",ac:"Rikas tekstieditori",ad:"Peru",ae:"Tee uudelleen",af:"Tallenna",ag:"Peruuta",ah:"Vaihtoehtoinen teksti",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));