(function(d){d['nb']=Object.assign(d['nb']||{},{a:"Kan ikke laste opp fil:",b:"Blokksitat",c:"Kursiv",d:"Fet",e:"Bilde-widget",f:"Bilde i full størrelse",g:"Sidebilde",h:"Venstrejustert bilde",i:"Midtstilt bilde",j:"Høyrejustert bilde",k:"Skriv inn bildetekst",l:"Sett inn bilde",m:"Lenke",n:"Opplasting feilet",o:"Velg overskrift",p:"Overskrift",q:"Nummerert liste",r:"Punktmerket liste",s:"Endre tekstalternativ for bilde",t:"Avsnitt",u:"Overskrift 1",v:"Overskrift 2",w:"Overskrift 3",x:"Fjern lenke",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Rikteksteditor, %0",ac:"Rikteksteditor",ad:"Angre",ae:"Gjør om",af:"Lagre",ag:"Avbryt",ah:"Tekstalternativ for bilde",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));