(function(d){d['ja']=Object.assign(d['ja']||{},{a:"ファイルをアップロードできません:",b:"ブロッククオート(引用)",c:"イタリック",d:"ボールド",e:"画像ウィジェット",f:"フルサイズ画像",g:"サイドイメージ",h:"Left aligned image",i:"Centered image",j:"Right aligned image",k:"画像の注釈を入力",l:"画像挿入",m:"リンク",n:"アップロード失敗",o:"見出しを選択",p:"見出し",q:"番号付きリスト",r:"箇条書きリスト",s:"画像の代替テキストを変更",t:"パラグラフ",u:"見出し1",v:"見出し2",w:"見出し3 ",x:"リンク解除",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"リッチテキストエディター, %0",ac:"リッチテキストエディター",ad:"元に戻す",ae:"やり直し",af:"保存",ag:"キャンセル",ah:"代替テキスト",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));