(function(d){d['fr']=Object.assign(d['fr']||{},{a:"Envoi du fichier échoué :",b:"Citation",c:"Italique",d:"Gras",e:"Objet image",f:"Image taille réelle",g:"Image sur le côté",h:"Image alignée à gauche",i:"Image centrée",j:"Image alignée a droite",k:"Saisissez la légende de l’image",l:"Insérer une image",m:"Lien",n:"Échec de l'envoi",o:"Choisir le titre",p:"En-tête",q:"Liste numérotée",r:"Liste à puces",s:"Changer le texte alternatif à l’image",t:"Paragraphe",u:"Titre 1",v:"Titre 2",w:"Titre 3",x:"Supprimer le lien",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Éditeur de texte enrichi, %0",ac:"Éditeur de texte enrichi",ad:"Annuler",ae:"Restaurer",af:"Enregistrer",ag:"Annuler",ah:"Texte alternatif",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));