(function(d){d['eo']=Object.assign(d['eo']||{},{a:"Cannot upload file:",b:"Block quote",c:"kursiva",d:"grasa",e:"bilda fenestraĵo",f:"Bildo kun reala dimensio",g:"Flanka biildo",h:"Left aligned image",i:"Centered image",j:"Right aligned image",k:"Skribu klarigon pri la bildo",l:"Enmetu bildon",m:"Ligilo",n:"Upload failed",o:"Elektu ĉapon",p:"Ĉapo",q:"Numerita Listo",r:"Bula Listo",s:"Ŝanĝu la alternativan tekston de la bildo",t:"Paragrafo",u:"Ĉapo 1",v:"Ĉapo 2",w:"Ĉapo 3",x:"Malligi",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Redaktilo de Riĉa Teksto, %0",ac:"Redaktilo de Riĉa Teksto",ad:"Malfari",ae:"Refari",af:"Konservi",ag:"Nuligi",ah:"Alternativa teksto",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));