(function(d){d['kn']=Object.assign(d['kn']||{},{a:"Cannot upload file:",b:"‍‍‍‍ಗುರುತಿಸಲಾದ ‍‍ಉಲ್ಲೇಖ",c:"‍ಇಟಾಲಿಕ್",d:"‍‍ದಪ್ಪ",e:"‍ಚಿತ್ರ ವಿಜೆಟ್",f:"‍ಪೂರ್ಣ ‍‍ಅಳತೆಯ ಚಿತ್ರ",g:"‍ಪಕ್ಕದ ಚಿತ್ರ",h:"Left aligned image",i:"Centered image",j:"Right aligned image",k:"‍ಚಿತ್ರದ ಶೀರ್ಷಿಕೆ ಸೇರಿಸು",l:"Insert image",m:"‍ಕೊಂಡಿ",n:"Upload failed",o:"ಶೀರ್ಷಿಕೆ ಆಯ್ಕೆಮಾಡು",p:"ಶೀರ್ಷಿಕೆ",q:"‍ಸಂಖ್ಯೆಯ ಪಟ್ಟಿ‍",r:"‍‍ಬುಲೆಟ್ ಪಟ್ಟಿ",s:"‍ಚಿತ್ರದ ಬದಲಿ ಪಠ್ಯ ಬದಲಾಯಿಸು",t:"ಪ್ಯಾರಾಗ್ರಾಫ್",u:"ಶೀರ್ಷಿಕೆ 1",v:"ಶೀರ್ಷಿಕೆ 2",w:"ಶೀರ್ಷಿಕೆ 3",x:"‍ಕೊಂಡಿ ತೆಗೆ",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"‍ಸಮೃದ್ಧ ಪಠ್ಯ ಸಂಪಾದಕ‍, %0",ac:"‍ಸಮೃದ್ಧ ಪಠ್ಯ ಸಂಪಾದಕ‍‍",ad:"‍‍ರದ್ದು",ae:"‍ಮತ್ತೆ ಮಾಡು",af:"ಉಳಿಸು",ag:"ರದ್ದುಮಾಡು",ah:"‍ಪಠ್ಯದ ಬದಲಿ",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));