Please don't edit this filename
It is just for babel
generated from /admin/setting-admin/get_babel_string/
{{_("Create")}}
{{_("Current Domain")}}

{{_("domain.cdn")}} {{_("Create")}} {{_("Save & Add More")}} {{_("Close")}} {{_("Back")}}
{{_("config.admin_lang.label")}}
{{_("config.admin_lang.description")}}{{_("config.admin.label")}}{{_("config.admin.description")}}{{_("config.lang.label")}}
{{_("config.lang.description")}}{{_("config.branding_title.label")}}
{{_("config.branding_title.description")}}{{_("config.branding_site.label")}}
{{_("config.branding_site.description")}}{{_("config.branding_freetext.label")}}
{{_("config.branding_freetext.description")}}{{_("config.branding.label")}}{{_("config.branding.description")}}{{_("config.firewall.label")}}
{{_("config.firewall.description")}}{{_("config.auto_update.label")}}
{{_("config.auto_update.description")}}{{_("config.speed_test.label")}}
{{_("config.speed_test.description")}}{{_("config.only_ipv4.label")}}
{{_("config.only_ipv4.description")}}{{_("config.show_usage_in_sublink.label")}}
{{_("config.show_usage_in_sublink.description")}}{{_("config.decoy_domain.label")}}
{{_("config.decoy_domain.description")}}{{_("config.country.label")}}
{{_("config.country.description")}}{{_("config.general.label")}}{{_("config.general.description")}}{{_("config.block_iran_sites.label")}}
{{_("config.block_iran_sites.description")}}{{_("config.vmess_enable.label")}}
{{_("config.vmess_enable.description")}}{{_("config.v2ray_enable.label")}}
{{_("config.v2ray_enable.description")}}{{_("config.shared_secret.label")}}
{{_("config.shared_secret.description")}}{{_("config.proxies.label")}}{{_("config.proxies.description")}}{{_("config.domain_fronting_tls_enable.label")}}
{{_("config.domain_fronting_tls_enable.description")}}{{_("config.domain_fronting_domain.label")}}
{{_("config.domain_fronting_domain.description")}}{{_("config.domain_fronting.label")}}{{_("config.domain_fronting.description")}}{{_("config.telegram_enable.label")}}
{{_("config.telegram_enable.description")}}{{_("config.telegram_adtag.label")}}
{{_("config.telegram_adtag.description")}}{{_("config.telegram_fakedomain.label")}}
{{_("config.telegram_fakedomain.description")}}{{_("config.telegram_lib.label")}}
{{_("config.telegram_lib.description")}}{{_("config.telegram.label")}}{{_("config.telegram.description")}}{{_("config.http_proxy_enable.label")}}
{{_("config.http_proxy_enable.description")}}{{_("config.http_ports.label")}}
{{_("config.http_ports.description")}}{{_("config.http.label")}}{{_("config.http.description")}}{{_("config.allow_invalid_sni.label")}}
{{_("config.allow_invalid_sni.description")}}{{_("config.tls_ports.label")}}
{{_("config.tls_ports.description")}}{{_("config.tls.label")}}{{_("config.tls.description")}}{{_("config.ssfaketls_enable.label")}}
{{_("config.ssfaketls_enable.description")}}{{_("config.ssfaketls_fakedomain.label")}}
{{_("config.ssfaketls_fakedomain.description")}}{{_("config.ssfaketls.label")}}{{_("config.ssfaketls.description")}}{{_("config.shadowtls_enable.label")}}
{{_("config.shadowtls_enable.description")}}{{_("config.shadowtls_fakedomain.label")}}
{{_("config.shadowtls_fakedomain.description")}}{{_("config.shadowtls.label")}}{{_("config.shadowtls.description")}}{{_("config.restls.label")}}{{_("config.restls.description")}}{{_("config.tuic.label")}}{{_("config.tuic.description")}}{{_("config.ssr.label")}}{{_("config.ssr.description")}}{{_("config.kcp.label")}}{{_("config.kcp.description")}}{{_("config.utls.label")}}
{{_("config.utls.description")}}{{_("config.telegram_bot_token.label")}}
{{_("config.telegram_bot_token.description")}}{{_("config.package_mode.label")}}
{{_("config.package_mode.description")}}{{_("config.advanced.label")}}{{_("config.advanced.description")}}{{_("config.proxy_path.label")}}
{{_("config.proxy_path.description")}}{{_("config.path_trojan.label")}}
{{_("config.path_trojan.description")}}{{_("config.path_vless.label")}}
{{_("config.path_vless.description")}}{{_("config.path_vmess.label")}}
{{_("config.path_vmess.description")}}{{_("config.path_grpc.label")}}
{{_("config.path_grpc.description")}}{{_("config.path_tcp.label")}}
{{_("config.path_tcp.description")}}{{_("config.path_ws.label")}}
{{_("config.path_ws.description")}}{{_("config.cloudflare.label")}}
{{_("config.cloudflare.description")}}{{_("config.too_advanced.label")}}{{_("config.too_advanced.description")}}{{_("config.warp_plus_code.label")}}
{{_("config.warp_plus_code.description")}}{{_("config.warp_mode.label")}}
{{_("config.warp_mode.description")}}{{_("config.warp.label")}}{{_("config.warp.description")}}{{_("config.reality_short_ids.label")}}
{{_("config.reality_short_ids.description")}}{{_("config.reality_fallback_domain.label")}}
{{_("config.reality_fallback_domain.description")}}{{_("config.reality_server_names.label")}}
{{_("config.reality_server_names.description")}}{{_("config.reality_private_key.label")}}
{{_("config.reality_private_key.description")}}{{_("config.reality_public_key.label")}}
{{_("config.reality_public_key.description")}}{{_("config.reality.label")}}{{_("config.reality.description")}}{{_("config.first_setup.label")}}
{{_("config.first_setup.description")}}{{_("config.warp_enable.label")}}
{{_("config.warp_enable.description")}}{{_("config.warp_plus_code.label")}}
{{_("config.warp_plus_code.description")}}{{_("config.warp_mode.label")}}
{{_("config.warp_mode.description")}}{{_("config.reality_fallback_domain.label")}}
{{_("config.reality_fallback_domain.description")}}{{_("config.reality_server_names.label")}}
{{_("config.reality_server_names.description")}}{{_("config.reality_short_ids.label")}}
{{_("config.reality_short_ids.description")}}{{_("config.reality_private_key.label")}}
{{_("config.reality_private_key.description")}}{{_("config.reality_public_key.label")}}
{{_("config.reality_public_key.description")}}{{_("config.restls1_2_domain.label")}}
{{_("config.restls1_2_domain.description")}}{{_("config.restls1_3_domain.label")}}
{{_("config.restls1_3_domain.description")}}{{_("config.show_usage_in_sublink.label")}}
{{_("config.show_usage_in_sublink.description")}}{{_("config.cloudflare.label")}}
{{_("config.cloudflare.description")}}{{_("config.license.label")}}
{{_("config.license.description")}}{{_("config.country.label")}}
{{_("config.country.description")}}{{_("config.package_mode.label")}}
{{_("config.package_mode.description")}}{{_("config.utls.label")}}
{{_("config.utls.description")}}{{_("config.telegram_bot_token.label")}}
{{_("config.telegram_bot_token.description")}}{{_("config.is_parent.label")}}
{{_("config.is_parent.description")}}{{_("config.parent_panel.label")}}
{{_("config.parent_panel.description")}}{{_("config.unique_id.label")}}
{{_("config.unique_id.description")}}{{_("config.last_hash.label")}}
{{_("config.last_hash.description")}}{{_("config.cdn_forced_host.label")}}
{{_("config.cdn_forced_host.description")}}{{_("config.lang.label")}}
{{_("config.lang.description")}}{{_("config.admin_lang.label")}}
{{_("config.admin_lang.description")}}{{_("config.admin_secret.label")}}
{{_("config.admin_secret.description")}}{{_("config.tls_ports.label")}}
{{_("config.tls_ports.description")}}{{_("config.http_ports.label")}}
{{_("config.http_ports.description")}}{{_("config.kcp_ports.label")}}
{{_("config.kcp_ports.description")}}{{_("config.kcp_enable.label")}}
{{_("config.kcp_enable.description")}}{{_("config.decoy_domain.label")}}
{{_("config.decoy_domain.description")}}{{_("config.proxy_path.label")}}
{{_("config.proxy_path.description")}}{{_("config.firewall.label")}}
{{_("config.firewall.description")}}{{_("config.netdata.label")}}
{{_("config.netdata.description")}}{{_("config.http_proxy_enable.label")}}
{{_("config.http_proxy_enable.description")}}{{_("config.block_iran_sites.label")}}
{{_("config.block_iran_sites.description")}}{{_("config.allow_invalid_sni.label")}}
{{_("config.allow_invalid_sni.description")}}{{_("config.auto_update.label")}}
{{_("config.auto_update.description")}}{{_("config.speed_test.label")}}
{{_("config.speed_test.description")}}{{_("config.only_ipv4.label")}}
{{_("config.only_ipv4.description")}}{{_("config.shared_secret.label")}}
{{_("config.shared_secret.description")}}{{_("config.telegram_enable.label")}}
{{_("config.telegram_enable.description")}}{{_("config.telegram_adtag.label")}}
{{_("config.telegram_adtag.description")}}{{_("config.telegram_lib.label")}}
{{_("config.telegram_lib.description")}}{{_("config.telegram_fakedomain.label")}}
{{_("config.telegram_fakedomain.description")}}{{_("config.v2ray_enable.label")}}
{{_("config.v2ray_enable.description")}}{{_("config.torrent_block.label")}}
{{_("config.torrent_block.description")}}{{_("config.ssfaketls_enable.label")}}
{{_("config.ssfaketls_enable.description")}}{{_("config.ssfaketls_fakedomain.label")}}
{{_("config.ssfaketls_fakedomain.description")}}{{_("config.shadowtls_enable.label")}}
{{_("config.shadowtls_enable.description")}}{{_("config.shadowtls_fakedomain.label")}}
{{_("config.shadowtls_fakedomain.description")}}{{_("config.tuic_enable.label")}}
{{_("config.tuic_enable.description")}}{{_("config.tuic_port.label")}}
{{_("config.tuic_port.description")}}{{_("config.ssr_enable.label")}}
{{_("config.ssr_enable.description")}}{{_("config.ssr_fakedomain.label")}}
{{_("config.ssr_fakedomain.description")}}{{_("config.vmess_enable.label")}}
{{_("config.vmess_enable.description")}}{{_("config.domain_fronting_domain.label")}}
{{_("config.domain_fronting_domain.description")}}{{_("config.domain_fronting_http_enable.label")}}
{{_("config.domain_fronting_http_enable.description")}}{{_("config.domain_fronting_tls_enable.label")}}
{{_("config.domain_fronting_tls_enable.description")}}{{_("config.db_version.label")}}
{{_("config.db_version.description")}}{{_("config.branding_title.label")}}
{{_("config.branding_title.description")}}{{_("config.branding_site.label")}}
{{_("config.branding_site.description")}}{{_("config.branding_freetext.label")}}
{{_("config.branding_freetext.description")}}{{_("config.not_found.label")}}
{{_("config.not_found.description")}}{{_("config.path_vmess.label")}}
{{_("config.path_vmess.description")}}{{_("config.path_vless.label")}}
{{_("config.path_vless.description")}}{{_("config.path_trojan.label")}}
{{_("config.path_trojan.description")}}{{_("config.path_v2ray.label")}}
{{_("config.path_v2ray.description")}}{{_("config.path_ss.label")}}
{{_("config.path_ss.description")}}{{_("config.path_ws.label")}}
{{_("config.path_ws.description")}}{{_("config.path_tcp.label")}}
{{_("config.path_tcp.description")}}{{_("config.path_grpc.label")}} {{_("config.path_grpc.description")}}