@font-face {
    font-family: Vazir;
    src: url('../fonts/Vazir.eot');
    src: url('../fonts/Vazir.eot?#iefix') format('embedded-opentype'),
    url('../fonts/Vazir.woff2') format('woff2'),
    url('../fonts/Vazir.woff') format('woff'),
    url('../fonts/Vazir.ttf') format('truetype');
    font-weight: normal;
}

@font-face {
    font-family: Vazir;
    src: url('../fonts/Vazir-Bold.eot');
    src: url('../fonts/Vazir-Bold.eot?#iefix') format('embedded-opentype'),
    url('../fonts/Vazir-Bold.woff2') format('woff2'),
    url('../fonts/Vazir-Bold.woff') format('woff'),
    url('../fonts/Vazir-Bold.ttf') format('truetype');
    font-weight: bold;
}

@font-face {
    font-family: Vazir;
    src: url('../fonts/Vazir-Light.eot');
    src: url('../fonts/Vazir-Light.eot?#iefix') format('embedded-opentype'),
    url('../fonts/Vazir-Light.woff2') format('woff2'),
    url('../fonts/Vazir-Light.woff') format('woff'),
    url('../fonts/Vazir-Light.ttf') format('truetype');
    font-weight: 300;
}

@font-face {
    font-family: Vazir;
    src: url('../fonts/Vazir-Medium.eot');
    src: url('../fonts/Vazir-Medium.eot?#iefix') format('embedded-opentype'),
    url('../fonts/Vazir-Medium.woff2') format('woff2'),
    url('../fonts/Vazir-Medium.woff') format('woff'),
    url('../fonts/Vazir-Medium.ttf') format('truetype');
    font-weight: 500;
}

@font-face {
    font-family: Vazir;
    src: url('../fonts/Vazir-Thin.eot');
    src: url('../fonts/Vazir-Thin.eot?#iefix') format('embedded-opentype'),
    url('../fonts/Vazir-Thin.woff2') format('woff2'),
    url('../fonts/Vazir-Thin.woff') format('woff'),
    url('../fonts/Vazir-Thin.ttf') format('truetype');
    font-weight: 100;
}

@font-face {
    font-family: Vazir;
    src: url('../fonts/Vazir-Black.eot');
    src: url('../fonts/Vazir-Black.eot?#iefix') format('embedded-opentype'),
    url('../fonts/Vazir-Black.woff2') format('woff2'),
    url('../fonts/Vazir-Black.woff') format('woff'),
    url('../fonts/Vazir-Black.ttf') format('truetype');
    font-weight: 900;
}

/*---------------------------------- End of fonts ---------------------------------- */

body {
    font-family: 'Vazir', sans-serif !important;
    text-align:justify;
    
}

.ltr {
    direction:ltr;
}


/* ul {
    padding-inline-start: 0px;
} */

.dropdown-item {
    text-align: right;
}
.dropdown-footer, .dropdown-header {
    text-align: center;
}

.breadcrumb-item+.breadcrumb-item::before {
    padding-left: .5rem;
}


/*------------------------------------ contacts-list ------------------------------------*/

.contacts-list-img {
    float: right;
}

.contacts-list-info {
    margin-right: 45px;
    margin-left: 0;
}


/* sidebar */

.sidebar {
    direction: ltr;
}

.sidebar > div:first-child {
    direction: rtl;
}

.control-sidebar, .control-sidebar:before {
    left: -250px;
    right: auto;
    transition: left .3s ease-in-out;
}

.control-sidebar-slide-open .control-sidebar, .control-sidebar-slide-open .control-sidebar:before {
    left: 0;
    right: auto;
}

.small-box .icon {
    right:auto;
    left:10px;
}
.small-box .icon > i{
    right:auto;
    left:15px;
}

.nav-sidebar .nav-link>p>.right {
    left: 1rem;
    right: auto;
}

.nav-sidebar>.nav-item .nav-icon {
    margin-right: 0;
    margin-left: .2rem;
}

@media (min-width: 992px) {
    .sidebar-mini.sidebar-collapse .content-wrapper, .sidebar-mini.sidebar-collapse .main-footer, .sidebar-mini.sidebar-collapse .main-header {
        margin-right: 4.6rem;
        margin-left: 0;
    }
}

@media (min-width: 992px) {
    .sidebar-mini.sidebar-collapse .content-wrapper, .sidebar-mini.sidebar-collapse .main-footer, .sidebar-mini.sidebar-collapse .main-header {
        margin-left: 0rem !important;
        margin-right: 4.6rem !important;
        z-index:100;
    }

    .sidebar-mini.sidebar-collapse .main-sidebar, .sidebar-mini.sidebar-collapse .main-sidebar:before {
        margin-right: 0;
        width: 4.6rem;
    }
}


@media (min-width: 768px) {
    .content-wrapper, .main-footer, .main-header {
        transition: margin-right .3s ease-in-out;
        margin-right: 250px;
        margin-left : 0;
        z-index: 500;
    }
}

@media (max-width: 991.98px) {
    .main-sidebar, .main-sidebar:before {
        box-shadow: none!important;
        margin-right: -250px;
        margin-left: 0;
    }

    .content-wrapper, .content-wrapper:before, .main-footer, .main-footer:before, .main-header, .main-header:before {
        margin-right: 0;
    }

    .sidebar-open .main-sidebar, .sidebar-open .main-sidebar:before {
        margin-right: 0;
        z-index: 4000;
    }
}

.sidebar-collapse .main-sidebar, .sidebar-collapse .main-sidebar:before {
    margin-right: -250px;
}

@media (min-width: 768px) {
    .sidebar-collapse .content-wrapper, .sidebar-collapse
    .main-footer, .sidebar-collapse
    .main-header {
      margin-right: 0;
    }
  }

.main-sidebar {
    left: auto;
    right:0;
}

/* Cards */

.card-header>.card-tools {
    right:auto;
    left:1rem;
}

.user-block img {
     float: right;
}

.attachment-block .attachment-img {
    float: right;
}

.attachment-block .attachment-pushed {
    margin-left: 0px;
    margin-right: 110px;
}

.card-comments .card-comment img, .img-lg, .img-md, .img-sm, .user-block.user-block-sm img {
    float: right;
}

.card-comments .comment-text {
    margin-right: 40px;
    margin-left: 0;
}

.card-comments .card-comment img+.img-push, .img-sm+.img-push, .user-block.user-block-sm img+.img-push {
    margin-left: 0px;
    margin-right: 40px;
}

.user-block .comment, .user-block .description, .user-block .username {
    margin-right: 50px;
    margin-left: 0px;
}

.btn-group-vertical {
    align-items: center;
}


/*------------------------- Form ----------------------------*/
.select2-container .select2-search--inline {
    float: right;
}

.select2-container--default .select2-search--inline .select2-search__field {
    text-align: right;
}

/*----------------------- Extra Page ---------------------------*/
.error-page>.headline {
    float: right;
}

.error-page>.error-content {
    margin-right: 190px;
    margin-left: 0;
}

.lockscreen-image {
    left: auto;
    right:-10px;
}

.lockscreen-credentials {
    margin-left: 0px;
    margin-right: 70px;
}

.nav {
    padding-inline-start: 0px !important;
}

/*---------------------------------- Persian Date Picker ---------------------------------- */

.datepicker-plot-area {
    font-family: 'Vazir', sans-serif;
}

.callout{
    border-right: 5px solid #eee;
    border-left: 0px solid #eee;
}

.callout{
    border-right: 5px solid #eee;
    border-left: 0px solid #eee;
}
.callout.callout-danger {
    border-right-color: #bd2130;
  }
  
  .callout.callout-warning {
    border-right-color: #d39e00;
  }
  
  .callout.callout-info {
    border-right-color: #117a8b;
  }
  
  .callout.callout-success {
    border-right-color: #1e7e34;
  }
  


.card-footer{
    display: block;

}




.custom-switch {
    padding-right: 2.25rem;
  }
  
  .custom-switch .custom-control-label::before {
    right: -2.25rem;
    width: 1.75rem;
    pointer-events: all;
    border-radius: 0.5rem;
  }
  
  .custom-switch .custom-control-label::after {
    top: calc(0.25rem + 2px);
    right: calc(-2.25rem + 2px);
    width: calc(1rem - 4px);
    height: calc(1rem - 4px);
    background-color: #adb5bd;
    border-radius: 0.5rem;
    transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-transform 0.15s ease-in-out;
    transition: transform 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
    transition: transform 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-transform 0.15s ease-in-out;
  }
  
  
  @media (prefers-reduced-motion: reduce) {
    .custom-switch .custom-control-label::after {
      transition: none;
    }
  }
  
  .custom-switch .custom-control-input:checked ~ .custom-control-label::after {
    background-color: #fff;
    -webkit-transform: translateX(-0.75rem);
    transform: translateX(-0.75rem);
  }
  
  .custom-switch .custom-control-input:disabled:checked ~ .custom-control-label::before {
    background-color: rgba(0, 123, 255, 0.5);
  }


  /* *{
    direction: rtl !important;
} */
.layout-fixed .main-sidebar {
    right: 0;
}
.brand-image {
    float: right;
}
.content-wrapper, .main-footer, .main-header {
    margin-left: 0px !important;
    margin-right: 250px;
}
.mr-auto-navbav{
    margin-right: auto!important;
}
.navbar-expand .navbar-nav .nav-link {
    padding-right: 1rem;
    padding-left: 1rem;
}

[class*=icheck-]>input:first-child:checked+input[type=hidden]+label::after, 
[class*=icheck-]>input:first-child:checked+label::after {
    right: 15px;
    left: auto;
}
.nav-sidebar .nav-link>.right, 
.nav-sidebar .nav-link>p>.right {
    left: 1rem;
    right: auto;
}
.nav-sidebar .nav-link>.right:nth-child(2), 
.nav-sidebar .nav-link>p>.right:nth-child(2) {
    left: 2.2rem;
    right: auto;
}
.small-box .icon>i {
    left: 15px;
    right: auto;
}
@media (min-width: 992px){
    .sidebar-mini.sidebar-collapse .content-wrapper, .sidebar-mini.sidebar-collapse .main-footer, .sidebar-mini.sidebar-collapse .main-header {
        margin-right: 4.6rem!important;
    }
    .sidebar-mini.sidebar-collapse .content-wrapper, .sidebar-mini.sidebar-collapse .main-footer, .sidebar-mini.sidebar-collapse .main-header {
        margin-right: 4.6rem!important;
        margin-left: 0!important;
    }
}

@media (max-width: 767.98px) {
    .main-sidebar, .main-sidebar::before {
        box-shadow: none !important;
        margin-right: -250px;
    }

    .content-wrapper, .content-wrapper::before, .main-footer, .main-footer::before, .main-header, .main-header::before {
        margin-right: 0;
    }

    .sidebar-open .main-sidebar, .sidebar-open .main-sidebar::before {
        margin-right: 0;
    }
}

.card-title{
    float:right;
}
.card-header > .card-tools{
 float: left;   
}

.main-header .nav-link {
    text-align: right;
}
.nav-treeview{
    margin-left: 0px;
    margin-right: 20px;
}

.pull-right{
    float:left;
}
.pull-left{
    float:right;
}

.ml-auto, .mx-auto {
    margin-right: auto !important;
    margin-left: 0 !important;
}