
button.ColVis_Button,
ul.ColVis_collection li {
	padding: 0.5em;
}

ul.ColVis_collection {
	margin: 0;
	padding: 0;
	overflow: hidden;
	z-index: 2002;
}

ul.ColVis_collection li {
	clear: both;
	display: block;
	text-align: left;
	margin: -1px 0 0 0;
}

ul.ColVis_collection li span {
	display: inline-block;
	padding-left: 0.5em;
	cursor: pointer;
}

div.ColVis_collectionBackground {
	position: fixed;
	top: 0;
	left: 0;
	height: 100%;
	width: 100%;
	background-color: black;
	z-index: 1100;
}


div.ColVis_catcher {
	position: absolute;
	z-index: 1101;
}