(function(d){d['pl']=Object.assign(d['pl']||{},{a:"Nie można przesłać pliku:",b:"Cytat blokowy",c:"Kursywa",d:"Pogrubienie",e:"image widget",f:"Full size image",g:"Side image",h:"Left aligned image",i:"Centered image",j:"Right aligned image",k:"Enter image caption",l:"Wstaw obraz",m:"Wstaw odnośnik",n:"Upload failed",o:"Wybierz nagłówek",p:"Nagłówek",q:"Lista numerowana",r:"Lista wypunktowana",s:"Change image text alternative",t:"Akapit",u:"Nagłówek 1",v:"Nagłówek 2",w:"Nagłówek 3",x:"Usuń odnośnik",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Rich Text Editor, %0",ac:"Rich Text Editor",ad:"Cofnij",ae:"Ponów",af:"Zapisz",ag:"Anuluj",ah:"Text alternative",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));