(function(d){d['el']=Object.assign(d['el']||{},{a:"Cannot upload file:",b:"Περιοχή παράθεσης",c:"Πλάγια",d:"Έντονη",e:"image widget",f:"Εικόνα πλήρης μεγέθους",g:"Side image",h:"Left aligned image",i:"Centered image",j:"Right aligned image",k:"Λεζάντα",l:"Εισαγωγή εικόνας",m:"Σύνδεσμος",n:"Upload failed",o:"Επιλέξτε κεφαλίδα",p:"Κεφαλίδα",q:"Αριθμημένη λίστα",r:"Λίστα κουκκίδων",s:"Αλλαγή εναλλακτικού κείμενου",t:"Παράγραφος",u:"Κεφαλίδα 1",v:"Κεφαλίδα 2",w:"Κεφαλίδα 3",x:"Αφαίρεση συνδέσμου",y:"Edit link",z:"Open link in new tab",aa:"This link has no URL",ab:"Επεξεργαστής Πλούσιου Κειμένου, 0%",ac:"Επεξεργαστής Πλούσιου Κειμένου",ad:"Αναίρεση",ae:"Επανάληψη",af:"Αποθήκευση",ag:"Ακύρωση",ah:"Εναλλακτικό κείμενο",ai:"Link URL"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));