{% from 'bootstrap4/form.html' import render_form %}
{% extends "master.html" %}
{% block title %}{{hconfig(ConfigEnum.branding_title) or _('user.home.title')}}{% endblock %}

{% block body_header %}



{%endblock%}

{% block body %}



<div class="container col-12 col-md-6 offset-md-3">
    <h1>{{hconfig(ConfigEnum.branding_title) or _('user.home.title')}}</h1>
    <div class="card-group login-container">
        <!-- Left Card: Login Form -->
        <div class="card card-outline card-primary ">
            <div class="card-header text-center">
                Login
            </div>
            <div class="card-body">
                {{render_form(form)}}
            </div>
        </div>
        {%if 0 and hconfig(ConfigEnum.proxy_path_client)==g.proxy_path %}
        <!-- Right Card: Free Test Account -->
        <div class="card card-outline card-secondary">
            <div class="card-header text-center">
                
Free Test Account
            </div>
            <div class="card-body">
                
                <p class="card-text">Get a free test account with 200 MB and 7 days.</p>
                <button type="button" class="btn btn-success btn-block">Get Free Test Account</button>
            </div>
        </div>
        {%endif%}
    </div>
</div>

{% endblock %}