(function(d){d['tr']=Object.assign(d['tr']||{},{a:"Dosya yüklenemedi:",b:"Alıntı",c:"İtalik",d:"Kalın",e:"Görsel Bileşeni",f:"Tam Boyut Görsel",g:"Yan Görsel",h:"Sola hizalı görsel",i:"Ortalı görsel",j:"Sağa hizalı görsel",k:"Resim Açıklaması Gir",l:"Görsel Ekle",m:"Bağlantı",n:"Yükleme başarsız",o:"Başlık tipi seç",p:"Başlık",q:"Numaralı Liste",r:"Simgeli Liste",s:"Görsel alternatif yazısını değiştir",t:"Paragraf",u:"1. Seviye Başlık",v:"2. Seviye Başlık",w:"3. Seviye Başlık",x:"Bağlantıyı kaldır",y:"Bağlantıyı değiştir",z:"Yeni sekmede aç",aa:"Bağlantı adresi yok",ab:"Zengin İÇerik Editörü, %0",ac:"Zengin İçerik Editörü",ad:"Geri al",ae:"İleri al",af:"Kaydet",ag:"İptal",ah:"Yazı alternatifi",ai:"Bağlantı Adresi"})})(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));