<div class="col">
    {{_("Please select one of the following applications:")}}


    {% macro hiddifyng_android_dl() -%}
    <div class="btn-group">
        <a href="/{{hconfigs[ConfigEnum.proxy_path]}}/gh/Hiddify/HiddifyNG/releases/latest/download/HiddifyNG.apk" class="btn btn-primary orig-link">{{_('download')}}</a>
    </div>
    {%- endmacro -%}
    {% macro v2rayng_android_dl() -%}
    <div class="btn-group">
        <a href="/{{hconfigs[ConfigEnum.proxy_path]}}/gh/2dust/v2rayNG/releases/download/1.8.5/v2rayNG_1.8.5.apk" class="btn btn-primary orig-link">{{_('download')}}</a>
    </div>
    {%- endmacro -%}

    {% macro hiddifyng_android_google_play() -%}

    <div class="btn-group">
        <a href="https://play.google.com/store/apps/details?id=ang.hiddify.com" class="btn btn-primary orig-link">{{_('install from google play')}}</a>
    </div>


    {%- endmacro -%}
    <details data-widget="collapse">
        <summary class="callout callout-success">
            {{_("HiddifyNG")}}
        </summary>

        {{_('HiddifyNG intro',dl_link=hiddifyng_android_dl(),play_link=hiddifyng_android_google_play())}}

        <div class="btn-group">
            <a href="hiddify://install-sub?url=https%3A%2F%2F{{domain}}%2F{{hconfigs[ConfigEnum.proxy_path]}}%2F{{user.uuid}}%2Fall.txt%3Fname%3D{{(db_domain.alias or db_domain.domain|urlencode)}}-{{asn}}%26asn%3D{{asn}}%26mode%3D{{mode}}"
                class="btn btn-light orig-link"> {{_('Click to Import')}} </a>
        </div>
        <details>
            <summary>{{_("V2rayNG")}}</summary>
            {{_('V2rayNG intro',dl_link=v2rayng_android_dl(),play_link="")}}
            <div class="btn-group">
                <a href="https://{{domain}}/{{hconfigs[ConfigEnum.proxy_path]}}/{{user.uuid}}/all.txt?name={{db_domain.alias or db_domain.domain}}-{{asn}}&asn={{asn}}&mode={{mode}}"
                    class="btn btn-light orig-link"> {{_('user.home.allconfig.link-configs')}} </a>
            </div>
        </details>
    </details>


    <details data-widget="collapse">
        <summary class="callout callout-success">
            {{_("Hiddify Android")}}
        </summary>





        {% macro hiddify_android_dl() -%}
        <div class="btn-group">
            <a href="/{{hconfigs[ConfigEnum.proxy_path]}}/gh/hiddify/HiddifyProxyAndroid/releases/download/v2.7.3-h0.24/hiddify-2.7.3-h0.24-meta-alpha-universal-release.apk"
                class="btn btn-primary orig-link">{{_('download')}} 13</a>
        </div>
        {%- endmacro -%}

        {% macro hiddify_android_google_play() -%}
        <!-- <div class="btn-group">
    <a href="/{{hconfigs[ConfigEnum.proxy_path]}}/gh/hiddify/HiddifyProxyAndroid/releases/download/v0.8/hiddify-2.6.1-h0.8-meta-alpha-universal-release.apk"
        class="btn btn-primary orig-link">{{_('install from google play')}}</a>
</div> -->
        {%- endmacro -%}

        {{_('user.home.android.intro',dl_link=hiddify_android_dl(),play_link=hiddify_android_google_play())}}



        <!--                                         

                        با استفاده از یکی از دو لینک زیر، اپلیکیشن کلش Clash را دانلود و نصب کنید. 
                        
                        <br>
                         -->

        {% with show_meta=True %}
        {% include "home/clash-links.html" %}
        {% endwith %}
        <!--
                        ابندا یکی از لینک تنظیمات کلش را کپی کنید و در بخش 2 مرحله 4 قرار دهید و مراحل را مطابق
                        گیف
                        زیر انجام دهید


                         <figure class="figure d-block text-center p-3">
                            <img src="images/clash_android.gif" alt="How to use clash for Android"
                                class="figure-img img-fluid rounded" />
                            <figcaption class="figure-caption">
                                <p>How to use clash for Android</p>
                            </figcaption>
                        </figure> -->

    </details>
</div>