{% extends "master.html" %}
{% block title %}{{hconfig(ConfigEnum.branding_title) or _('user.home.title')}}{% endblock %}
{% from 'macros.html' import modal %}
{% block tail_js %}
{{super()}}
{% macro copy_link() -%}
<a href="{{new_link}}" class='btn btn-success copy-link'>
    <i class="fa-regular fa-copy"></i> {{_("copy")}}
</a>

<a href="{{new_link}}" class='btn btn-primary'>
    <i class="fa-solid fa-arrow-up-right-from-square"></i> {{_("open")}}
</a>
{%- endmacro -%}

{{modal("link-alarm-modal",_("Link is changed!"),_("The page has been moved to a new link. Please copy and save the new link."),footer=copy_link(),show=True)}}


{% endblock %}